from typing import Any, Optional

import requests

try:
    from typing import Protocol
except ImportError:
    # Older python versions will not have Protocol available
    class Protocol:  # type: ignore
        pass


class ActiveSoupResult(Protocol):
    @property
    def url(self) -> str:
        ...

    def __getitem__(self, lookup: str) -> Any:
        ...


class Resolver(Protocol):
    def resolve(self, response: requests.Response) -> ActiveSoupResult:
        ...


class Driver(Protocol):
    def __enter__(self) -> "Driver":
        ...

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        ...

    def get(self, url, **kwargs) -> "Driver":
        ...

    @property
    def url(self) -> Optional[str]:
        ...

    def __getattr__(self, item: str) -> Any:
        ...

    def __getitem__(self, item) -> Any:
        ...



import requests

from activesoup.protocols import ActiveSoupResult
from activesoup.response import Response


class CsvResponse(Response):
    def __init__(self, raw_response):
        super().__init__(raw_response)
        self.content = raw_response.content

    def save(self, to):
        if isinstance(to, type("")):
            with open(to, "wb") as f:
                self._write_to_file(f)
        else:
            self._write_to_file(to)

    def _write_to_file(self, file_object):
        file_object.write(self.content)

    def __repr__(self) -> str:
        return "CsvResponse"

    def __str__(self) -> str:
        return "<[csv]>"


class Resolver:
    def resolve(self, raw_response: requests.Response) -> ActiveSoupResult:
        return CsvResponse(raw_response)
